package org.kie.examples.spring;

import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.container.test.api.TargetsContainer;
import org.jboss.shrinkwrap.api.Archive;
import org.jboss.shrinkwrap.api.ArchivePaths;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.jboss.shrinkwrap.resolver.api.maven.Maven;
import org.jboss.shrinkwrap.resolver.api.maven.strategy.CombinedStrategy;
import org.jboss.shrinkwrap.resolver.api.maven.strategy.RejectDependenciesStrategy;
import org.jboss.shrinkwrap.resolver.api.maven.strategy.TransitiveStrategy;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.kie.api.runtime.KieSession;
import org.springframework.context.support.AbstractApplicationContext;

import static org.junit.Assert.assertEquals;

public class KieSpringBeanTest extends AbstractSpringContainerTest {

    private static final String CTX_RESOURCE_NAME = "kieSessionTestContext.xml";
    private static final String DRL_RESOURCE_FULL_NAME = "/org/kie/examples/spring/kbase/test.drl";

    private AbstractApplicationContext ctx;
    private KieSession ksession;

    @Deployment
    @TargetsContainer(MANAGED_CONTAINER)
    public static Archive<?> deployManaged() {
        WebArchive war = ShrinkWrap.create(WebArchive.class, "arq-kie-spring-test.war");
        war.addAsLibraries(Maven.configureResolver()
                .workOffline()
                .loadPomFromFile("pom.xml","!wls-lib")
                .importRuntimeAndTestDependencies()
                .resolve()
                .using(new CombinedStrategy(
                        TransitiveStrategy.INSTANCE,
                        new RejectDependenciesStrategy(false,
                                                       "javax.enterprise:cdi-api",
                                                       "org.jboss.weld.se:weld-se-core",
                                                       "org.jboss.weld:weld-api",
                                                       "org.jboss.weld:weld-core",
                                                       "org.jboss.weld:weld-spi",
                                                       "org.jboss.seam.spring:seam-spring-core",
                                                       "org.slf4j:slf4j-ext")))
                .asFile());

        war.addPackages(true, KieSpringBeanTest.class.getPackage());
        war.addAsResource(KieSpringBeanTest.class.getPackage(), CTX_RESOURCE_NAME);
        war.addAsResource(KieSpringBeanTest.class.getResource(DRL_RESOURCE_FULL_NAME),
                          ArchivePaths.create(DRL_RESOURCE_FULL_NAME));

        return war;
    }

    @Before
    public void prepare() {
        ctx = getApplicationContext(this.getClass(), CTX_RESOURCE_NAME);
    }

    @After
    public void cleanup() {
        if (ksession != null) {
            ksession.dispose();
        }
        if (ctx != null) {
            ctx.close();
        }
    }

    @Test
    public void testStatefulKieSessionBean() {
        ksession = ctx.getBean("statefulSession", KieSession.class);
        ksession.insert("some string");
        final int fired = ksession.fireAllRules();
        System.out.println("Rules fired: " + fired);
        assertEquals("Exactly 1 rule from Spring bean should have fired.", 1, fired);
    }

}
